/*
 * $Id: config.h,v 1.3 1994/01/31 08:39:13 kayvan Exp $
 *
 * Config file for nbs_time.
 */

#define LOG	"/usr/adm/clocklog"

#define CLOCKPATH "/etc/clock"

#ifdef SET_UTC
#define CLOCKARGS "-w", "-u"
#else
#define CLOCKARGS "-w"
#endif

#define CU_PROG "/usr/bin/cu"
#define CU_ARGS "-s", "1200", "-c", NBS_NUM

/* Everything below this line should not need to be changed */

char *use_rcs_id = &rcs_id[0];	/* suppress unused warning for rcs_id */

/* Prototypes for functions used in nbs_time */

void time_stamp(time_t);
void set_time(), set_rtc(), disc_nbs();

/* Global variables */

FILE *Nbs_rd;	/* file descriptor for reading from NBS */
FILE *Nbs_wt;	/* file descriptor for writing to NBS   */
